using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Globalization;
using System.Windows;
using System.Windows.Input;

namespace TestMainWindow
{
    public class NcCalendarDay:INotifyPropertyChanged
    {
        public static readonly NcCalendarDay Empty = new NcCalendarDay();
        private DateTime mValue = new DateTime(1, 1, 1);
        private NcCalendar mOwner;
        private NcCalendarDay(){}
        private bool mIsPreviousMonth = false;
        private bool mIsNextMonth = false;

        public NcCalendarDay(DateTime oneDay)
        {
            mValue = oneDay;
        }

        public DateTime Value
        {
            get { return mValue; }
        }
        static NcCalendarDay()
        {
            CommandManager.RegisterClassCommandBinding(typeof(NcCalendarDay), new CommandBinding(Select, OnSelect));
        }

        #region Select
        /// <summary>
        /// ѡǰڡ
        /// </summary>
        public static RoutedUICommand Select { get { return mSelect; } }
        private static RoutedUICommand mSelect = new RoutedUICommand("Select", "Select", typeof(NcCalendarDay));

        private static void OnSelect(object sender, ExecutedRoutedEventArgs e)
        {
            Console.WriteLine("====");
            NcCalendarDay control = sender as NcCalendarDay;
            control.IsCurrent = true;
            if (control == null) return;
            // ʵִ
        }
        #endregion


        public int DayOfWeek { get { return (this.IsEmpty ? -1 : (int)mValue.DayOfWeek); } }

        public int Year { get { return (this.IsEmpty ? 0 : mValue.Year); } }

        public int Month { get { return (this.IsEmpty ? 0 : mValue.Month); } }

        public int Day { get { return (this.IsEmpty ? 0 : mValue.Day); } }

        public bool IsEmpty { get { return (this == NcCalendarDay.Empty); } }

        public bool IsToday { get { return mValue.Date == DateTime.Now.Date; } }

        public int WeekOfMonth
        {
            get
            {
                DateTime firstDay = new DateTime(mValue.Year, mValue.Month, 1);
                int emptyDays = (int)firstDay.DayOfWeek;
                return (int)Math.Ceiling((mValue.Day + emptyDays)/7f);
            }
        }

        public bool IsCurrent
        {
            get { return (mOwner != null && mOwner.Current == this); }
            set
            {
                if (mOwner == null) return;
                if (mOwner.Current == this) return;
                if (value)
                {
                    NcCalendarDay oldCurrent = mOwner.Current;
                    mOwner.Current = this;
                    OnPropertyChanged("IsCurrent");
                    if (oldCurrent != null) oldCurrent.OnPropertyChanged("IsCurrent");
                }
            }
        }

        public NcCalendar Owner
        {
            get { return mOwner; }
            internal set { this.mOwner = value; }
        }

        public bool IsPreviousMonth 
        {
            get { return mIsPreviousMonth; }
            internal set { mIsPreviousMonth = value; }
        }
        public bool IsNextMonth 
        {
            get { return mIsNextMonth; }
            internal set { mIsNextMonth = value; }
        }
        

        #region INotifyPropertyChanged Ա

        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Ըı¼
        /// </summary>
        /// <param name="propertyName">ơ</param>
        public void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventArgs args = new PropertyChangedEventArgs(propertyName);
            this.OnPropertyChanged(args);
        }
        /// <summary>
        /// Ըı¼
        /// </summary>
        /// <param name="e">¼</param>
        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (this.PropertyChanged != null) this.PropertyChanged(this, e);
        }
        #endregion
    }
}
